# Perl 
#
# Class Report::Porf::Framework
#
# Perl Open Report Framework (Porf)
#
# Framework to create/configure Reports for any output format.
#
# Ralf Peine, Mon Apr  7 08:07:00 2014
#
# More documentation at the end of file
#------------------------------------------------------------------------------

$VERSION = "1.000";

package Report::Porf;

use strict; 
use warnings;

use Carp;

use Report::Porf::Framework;

use base qw(Exporter);

our @EXPORT = qw();
our @EXPORT_OK = qw(auto_report create_report);
our %EXPORT_TAGS = (
		all => [qw(auto_report create_report)],
);				 

sub auto_report {
	return Report::Porf::Framework::auto_report(@_);
}

sub create_report {
	die "Waits for implementation!";
}


1;
__END__

=head1 NAME

Report::Porf

Perl Open Report Framework

Framework to create/configure Reports for any output format.

=head1 VERSION

This documentation refers to version 1.000 of Report::Porf

=head1 SYNOPSIS

   use Report::Porf qw(:all);

Best for data rows as hashes, also usable for data rows as arrays

   auto_report(\@data_rows);        # prints -max_rows => 10 to STDOUT
   auto_report(\@data_rows, $file); # prints all into file

Configure columns explicit for objects, hashes, arrays or other by

   my $report = create_report(...);

   $report->write_all(\@data_rows);        # prints out
   $report->write_all(\@data_rows, $file); # writes into file

Where

   my $file;

is filehandle or file name.

=head1 DESCRIPTION

=head2 Structure of a Table::Simple Report

  Report
    Table
      Line (Conatining data of data row)
        Cell
  
  *============+============+============*  # Bold separator line
  |   Prename  |   Surname  |     Age    |  # The header line
  *------------+------------+------------*  # Separator line
  | Vorname 1  | <a cell>   | 7.69230769 |  # A data line with <cell>s
  | Vorname 2  | Name 2     | 15.3846153 |
  | Vorname 3  | Name 3     | 23.0769230 |
  | Vorname 4  | Name 4     | 30.7692307 |
  *============+============+============*

=head2 Create Report

  my $report_frame_work = Report::Porf::Framework::Get();
  my $report            = $report_frame_work->CreateReport($format);

  # $report->SetVerbose(3); # uncomment to see infos about configuring phase

Current supported formats:

  HTML
  Text
  Csv

=head2 Configure Report

After creation a report has to be configured. 

Call C<ConfigureColumn(%options)> to configure a report. Following
options are available:

=head3 Layout

  -header  -h   constant: Text
  -align   -a   constant: (left|center|right)
                          (l   |   c  |    r)
  -width   -w   constant: integer
  -format  -f   constant: string for sprintf
  -color   -c   constant / sub {...}

The sub {...} makes conditional coloring easy possible.

As next, access to the value has to be defined. There are 4 alternatives
to get the value of a cell depending of type (array, hash, object).

=head3 GetValue Alternative 1 --- ARRAY

  my $prename = 1;
  my $surname = 2;
  my $age     = 3;

  $report->ConfigureColumn(-header => 'Prename', -value_indexed => $prename ); # long
  $report->ConfCol        (-h      => 'Surname', -val_idx       => $surname ); # short
  $report->CC             (-h      => 'Age',     -vi            => $age     ); # minimal

=head3 GetValue Alternative 2 --- HASH

  $report->ConfigureColumn(-header => 'Prename', -value_named => 'Prename' ); # long
  $report->ConfCol        (-h      => 'Surname', -val_nam     => 'Surname' ); # short
  $report->CC             (-h      => 'Age',     -vn          => 'Age'     ); # minimal

=head3 GetValue Alternative 3 --- OBJECT

  $report->ConfigureColumn(-header => 'Prename', -value_object => 'GetPrename()'); # long
  $report->ConfCol        (-h      => 'Surname', -val_obj      => 'GetSurname()'); # short
  $report->CC             (-h      => 'Age',     -vo           => 'GetAge()'    ); # minimal

=head3 GetValue Alternative 4 --- Free 

  $report->ConfigureColumn(-h => 'Prename',  -value =>    '"Dr. " . $_[0]->{Surname}'    );
  $report->ConfCol        (-h => 'Surname',    -val => sub { return $_[0]->{Prename}; }; );
  $report->CC             (-h => 'Age (Months)', -v =>     '(12.0 * $_[0]->GetAge())'    );

=head2 When All Columns Are Configured

  $report->ConfigureComplete(); 

=head2 Write Table Out Into File

  $report->WriteAll($person_rows, $out_file_name);

You can also put out single rows or single cells or start actions to do that.
[Needs to be explained more]

In "Report/Porf/examples" subdir you can find more examples.

=head1 Details

Here are the details for those, who want to modify an existing or
create a new ReportConfigurator. It's actually not complete. See
*ReportConfigurator.pm for more.

=head2 Report Attributes

There a following attributes of report, that can used by Get*/Set*

  FileStart
    PageStart
      TableStart
        *============+====  # BoldSeparatorLine
        |   Prename  | ...  # HeaderRowStart HeaderStart <HeaderText> HeaderEnd ... HeaderRowEnd
        *------------+----  # SeparatorLine
        | Vorname 1  | ...  # RowStart       CellStart   <CellValue>  CellEnd   ... RowEnd
        | ...        | ...  # ...
        *============+====  # BoldSeparatorLine
      TableEnd
    PageEnd
  FileEnd

To be continued...

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2014 by Ralf Peine, Germany.  All rights reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.6.0 or,
at your option, any later version of Perl 5 you may have available.

=head1 DISCLAIMER OF WARRANTY

This library is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=cut
