# perl
#
# Class Report::Porf::Table::Simple::CsvReportConfigurator
#
# Perl Open Report Framework (Porf)
#
# Configures a Report::Porf::Table::Simple to write out ASCII- or UTF-Csv tables
#
# Ralf Peine, Sat Feb 01 08:19:40 2014
#
# More documentation at the end of file
#------------------------------------------------------------------------------

$VERSION = "0.950";

#------------------------------------------------------------------------------
#
# Example list with 10 lines (csv)
#
# # Count,Prename,Surname,Age,TimeStamp
# 1,Vorname 1,Name 1,10,0.002329
# 2,Vorname 2,Name 2,20,0.003106
# 3,Vorname 3,Name 3,30,0.003822
# 4,Vorname 4,Name 4,40,0.004533
# 5,Vorname 5,Name 5,50,0.005235
# 6,Vorname 6,Name 6,60,0.005944
# 7,Vorname 7,Name 7,70,0.006656
# 8,Vorname 8,Name 8,80,0.007362
# 9,Vorname 9,Name 9,90,0.008069
# 10,Vorname 1,Name 10,100,0.008779
#
# # Time needed for export of 10 data lines: 0.001954
#
#------------------------------------------------------------------------------

use strict;
use warnings;

#--------------------------------------------------------------------------------
#
#  Report::Porf::Table::Simple::CsvReportConfigurator;
#
#--------------------------------------------------------------------------------

package Report::Porf::Table::Simple::CsvReportConfigurator;

use Carp;

use Report::Porf::Util;
use Report::Porf::Table::Simple;

#--------------------------------------------------------------------------------
#
#  Creation / Filling Of Instances
#
#--------------------------------------------------------------------------------

# --- Create Instance -----------------
sub new
{
    my $caller = $_[0];
    my $class  = ref($caller) || $caller;
    
    # let the class go
    my $self = {};
    bless $self, $class;

	$self->_init();

    return $self;
}

# --- Init ------------------------------------------------------------------

sub _init {
    my ($self        # instance_ref
        ) = @_;
	$self->SetDataSeparatorChar(',');
	$self->SetDataStringChar('"');
}

# --- Verbose ---------------------------------------------------------------

sub SetVerbose {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;
    
    $self->{Verbose} = $value;
}

sub GetVerbose {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Verbose};
}

# --- DataSeparatorChar ---------------------------------------------------------

sub SetDataSeparatorChar {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;
    
    $self->{DataSeparatorChar} = $value;
}

sub GetDataSeparatorChar {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{DataSeparatorChar};
}

# --- DataStringChar ---------------------------------------------------------

sub SetDataStringChar {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;
    
    $self->{DataStringChar} = $value;
}

sub GetDataStringChar {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{DataStringChar};
}

#--------------------------------------------------------------------------------
#
#  Configure format
#
#--------------------------------------------------------------------------------

# --- Create new report and configure -------------------------------------------
sub CreateAndConfigureReport {
    my ($self  # instance_ref
        ) = @_;

    return $self->ConfigureReport(Report::Porf::Table::Simple->new());
}

# --- Configure Export For Csv ------------------------------------------------
sub ConfigureReport {
    my ($self,                  # instance_ref
        $report2configure,      # report to be configured
        ) = @_;

    # $report2configure->SetDefaultColumnWidth (10);
    $report2configure->SetDefaultAlign       ('Left');

    $report2configure->SetFileStart('');
    $report2configure->SetFileEnd  ('');

    $report2configure->SetTableStart("");
    $report2configure->SetTableEnd  ("");

    $report2configure->SetRowStart("");
    $report2configure->SetRowEnd  ("\n");
    
    $report2configure->SetHeaderRowStart("# ");
    $report2configure->SetHeaderRowEnd  ("\n");
    $report2configure->SetHeaderStart("");
    $report2configure->SetHeaderEnd  ($self->GetDataSeparatorChar());
    
    $report2configure->SetCellStart("");
    $report2configure->SetCellEnd  ($self->GetDataSeparatorChar());

    #===================================================================================
    #
    #  Configure Actions, no need to speed up
    #
    #===================================================================================

    # === Column Store Action ============================================================
    $report2configure->SetConfigureColumnAction(
        sub {
            my $report  = shift; # instance_ref
            my %options = @_;    # options
            
            PrintHashRef(\%options) if Verbose($report, 2);
            
            my $cell_start = $report->GetCellStart();
            my $cell_end   = $report->GetCellEnd();

            my $left       = "";   # left from value
            my $right      = "";   # right from value

            # --- value ---------------------------------------
            my $value         = InterpreteValueOptions(\%options);
            my $value_ref     = ref($value);
            my $value_action;

            die "value action not defined" unless defined $value;
            
            if ($value_ref) {
                if ($value_ref =~ /^CODE/) {
                    $value_action = $value;
                }
                else {
                    # or what ??
                    die "# ref(value_ref) = $value_ref unknown";
                    # $value_action = eval ("sub { return $$value; };");
                }
            }
            else {
                $value_action = $report->CreateAction("return $value;");
            }

            # --- format value ----------------------------------------
            my $format = GetOptionValue(\%options, qw (-format -f));

            if ($format) {
                my $format_ref = ref($format);

                die "You used a '$format_ref' type to set the 'Format', "
                    ."but currently only strings are supported!"
                    if $format_ref;
                
                print "format $format\n" if Verbose($report, 3);

                $left   = "sprintf(\"$format\", ".$left;
                $right .= ")";
            }

            # --- coloring is not supported for csv ------------------
            my $color = GetOptionValue(\%options, qw (-color -c));

            print "color cannot be supported for csv (color = $color)\n" if Verbose($report, 3)  &&  $color;

            # --- width / align ---------------------------------------
            my $width = GetOptionValue(\%options, qw (-width -w));
            my $align = GetOptionValue(\%options, qw (-align -a));

            print "width cannot be supported for csv (width = $width)\n" if Verbose($report, 3)  &&  $width;
            print "align cannot be supported for csv (align = $align)\n" if Verbose($report, 3)  &&  $align;
            
            # --- configure header ----------------------------

            my $header_text = GetOptionValue(\%options, qw (-header -h));
            $header_text = '' unless defined $header_text;
            push (@{$report->GetHeaderTextsRef()}, $header_text);
            
            # --- ---------------------------------------------
            # --- ---------------------------------------------
            # --- escape separator / string char ----------------------------------------

			my $sep_char = $self->GetDataSeparatorChar();
			my $string_char = $self->GetDataStringChar();
			
			my $escape_sep_char_sub_ref = sub {
				my $value = $_[0];

				if ($value =~ /$sep_char|$string_char/) {
					$value =~ s/$string_char/$string_char$string_char/g;
					$value = "$string_char$value$string_char";
				}

				return $value;
			};
			
			$left   = '$escape_sep_char_sub_ref->('.$left;
			$right .= ")";
			
            # --- build cell content action ---------------------------------------------
            my $cell_action_str = $left.'$value_action->($_[0])'.$right;

            my $eval_str = 'sub { return $cell_start.'.$cell_action_str.'.$cell_end; }';

            print "### eval_str = sub { return $cell_start$cell_action_str$cell_end; }\n"
                if Verbose($report, 3);
            
            my $cell_action = eval ($eval_str);
            
            print "### ref(cell_action) ".ref($cell_action) ."\n" if Verbose($report, 3);
            
            $report->AddCellOutputAction($cell_action);
        });

    # === Configure Complete Action ====================================================
    $report2configure->SetConfigureCompleteAction(
        sub {
            my ($report,        # instance_ref
                ) = @_;

            $report->SetHeaderLine    ('');
            $report->SetSeparatorLine ('');
            $report->SetBoldHeaderLine('');
        });
    
	my $cell_end = $report2configure->GetCellEnd();

    #===================================================================================
    #
    #  Runtime Actions, no need to speed up
    #
    #===================================================================================

    # === Header Output Action ============================================================
    $report2configure->SetHeaderOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            my $header_string = $report->GetHeaderRowStart();
            my $c = 0;
            foreach my $header_text (@{$data_ref}) {
                $header_string .= $report->GetHeaderStart()
                    .$header_text
                    .$report->GetHeaderEnd();
                $c++;
            }

			$header_string =~ s/$cell_end$//;
            $header_string .= $report->GetHeaderRowEnd();

            print "### Header String:$header_string" if Verbose($report, 2);
            
            return $header_string;
        });

    # === Start Table Output Action ============================================================
    $report2configure->SetStartTableOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            return $report->GetFileStart()
                .$report->GetTableStart()
					.$report->GetHeaderOutput();
			
            
        });

    # === End Table Output Action ============================================================
    $report2configure->SetEndTableOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            return $report->GetTableEnd()
                .$report->GetFileEnd();
        });

    #===================================================================================
    #
    #  Runtime Mass Data Actions, be careful, don't slow it down !!
    #
    #===================================================================================

    my $previous_data_ref = '';

    # === Row Output Action ============================================================
    $report2configure->SetRowOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            my $cell_output_actions_ref = $report->GetCellOutputActions();

            my $row_string = '';

            # --- Add something when group changes ----
            my $row_group_changes_action = $report->GetRowGroupChangesAction();
            
            $row_string .= $row_group_changes_action->($previous_data_ref, $data_ref)
                if $row_group_changes_action;

            $previous_data_ref = $data_ref;

            # --- start new row -----------------------
            $row_string .= $report->GetRowStart();
            foreach my $action (@$cell_output_actions_ref) {
                print "### action $action\n" if Verbose($report, 4);
                $row_string .= $action->($data_ref);
            }

			$row_string =~ s/$cell_end$//;		
            $row_string .= $report->GetRowEnd();

            print "### Row String:$row_string" if Verbose($report, 2);
            
            return $row_string;
        });

    return $report2configure;
}
1;

=head1 NAME

C<Report::Porf::Table::Simple::CsvReportConfigurator>

Configures a Report::Porf::Table::Simple to write out Csv files.

Part of Perl Open Report Framework (Porf).

=head1 Documentation

See Report::Porf::Framework.pm for documentation of features and usage.

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2013 by Ralf Peine, Germany.  All rights reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.6.0 or,
at your option, any later version of Perl 5 you may have available.

=head1 DISCLAIMER OF WARRANTY

This library is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=cut
