# perl
#
# Class Report::Porf::Table::Simple::TextReportConfigurator
#
# Perl Open Report Framework (Porf)
#
# Configures a Report::Porf::Table::Simple to write out ASCII- or UTF-Text tables
#
# Ralf Peine, Sat Feb 01 08:21:26 2014
#
# More documentation at the end of file
#------------------------------------------------------------------------------

$VERSION = "0.950";

#------------------------------------------------------------------------------
#
# Example list with 10 lines (text)
#
# *============+======================+=========+=======+======================*
# @ Count      @ Prename              @ Surname @ Age   @ TimeStamp            @
# *------------+----------------------+---------+-------+----------------------*
# | 1          | Vorname 1            | Name 1  | 10    | 0.002329             |
# | 2          | Vorname 2            | Name 2  | 20    | 0.003106             |
# | 3          | Vorname 3            | Name 3  | 30    | 0.003822             |
# | 4          | Vorname 4            | Name 4  | 40    | 0.004533             |
# | 5          | Vorname 5            | Name 5  | 50    | 0.005235             |
# | 6          | Vorname 6            | Name 6  | 60    | 0.005944             |
# | 7          | Vorname 7            | Name 7  | 70    | 0.006656             |
# | 8          | Vorname 8            | Name 8  | 80    | 0.007362             |
# | 9          | Vorname 9            | Name 9  | 90    | 0.008069             |
# | 10         | Vorname 10           | Name 10 | 100   | 0.008779             |
# *============+======================+=========+=======+======================*
#
# # Time needed for export of 10 data lines: 0.001954
#
#------------------------------------------------------------------------------

use strict;
use warnings;

#--------------------------------------------------------------------------------
#
#  Report::Porf::Table::Simple::TextReportConfigurator;
#
#--------------------------------------------------------------------------------

package Report::Porf::Table::Simple::TextReportConfigurator;

use Carp;

use Report::Porf::Util;
use Report::Porf::Table::Simple;

#--------------------------------------------------------------------------------
#
#  Creation / Filling Of Instances
#
#--------------------------------------------------------------------------------

# --- Create Instance -----------------
sub new
{
    my $caller = $_[0];
    my $class  = ref($caller) || $caller;
    
    # let the class go
    my $self = {};
    bless $self, $class;

    return $self;
}

# --- Verbose ---------------------------------------------------------------

sub SetVerbose {
    my ($self,        # instance_ref
        $value        # value to set
        ) = @_;
    
    $self->{Verbose} = $value;
}

sub GetVerbose {
    my ($self,        # instance_ref
        ) = @_;
    
    return $self->{Verbose};
}

#--------------------------------------------------------------------------------
#
#  Configure format
#
#--------------------------------------------------------------------------------

# --- Create new report and configure -------------------------------------------
sub CreateAndConfigureReport {
    my ($self  # instance_ref
        ) = @_;

    return $self->ConfigureReport(Report::Porf::Table::Simple->new());
}

# --- Configure Export For Text ------------------------------------------------
sub ConfigureReport {
    my ($self,                  # instance_ref
        $report2configure,      # report to be configured
        ) = @_;

    $report2configure->SetFormat('Text');

    $report2configure->SetDefaultColumnWidth (10);
    $report2configure->SetDefaultAlign       ('Left');

    $report2configure->SetFileStart('');
    $report2configure->SetFileEnd  ('');

    $report2configure->SetTableStart("\n");
    $report2configure->SetTableEnd  ("\n");

    $report2configure->SetRowStart("|");
    $report2configure->SetRowEnd  ("\n");
    
    $report2configure->SetHeaderRowStart("|");
    $report2configure->SetHeaderRowEnd  ("\n");
    $report2configure->SetHeaderStart(" ");
    $report2configure->SetHeaderEnd  (" |");
    
    $report2configure->SetCellStart(" ");
    $report2configure->SetCellEnd  (" |");

    $report2configure->SetHorizontalSeparationStart           ("*");
    $report2configure->SetHorizontalSeparationColumnSeparator ("+");
    $report2configure->SetHorizontalSeparationEnd             ("*\n");
    $report2configure->SetHorizontalSeparationChar            ("-");
    $report2configure->SetHorizontalSeparationBoldChar        ("=");

    #===================================================================================
    #
    #  Configure Actions, no need to speed up
    #
    #===================================================================================

    # === Column Store Action ============================================================
    $report2configure->SetConfigureColumnAction(
        sub {
            my $report  = shift; # instance_ref
            my %options = @_;    # options
            
            PrintHashRef(\%options) if Verbose($report, 2);
            
            my $cell_start = $report->GetCellStart();
            my $cell_end   = $report->GetCellEnd();

            my $left       = "";   # left from value
            my $right      = "";   # right from value

            # --- value ---------------------------------------
            my $value         = InterpreteValueOptions(\%options);
            my $value_ref     = ref($value);
            my $value_action;

            die "value action not defined" unless defined $value;
            
            if ($value_ref) {
                if ($value_ref =~ /^CODE/) {
                    $value_action = $value;
                }
                else {
                    # or what ??
                    die "# ref(value_ref) = $value_ref unknown";
                    # $value_action = eval ("sub { return $$value; };");
                }
            }
            else {
                $value_action = $report->CreateAction("return $value;");
            }

            # --- format value ----------------------------------------
            my $format = GetOptionValue(\%options, qw (-format -f));

            if ($format) {
                my $format_ref = ref($format);

                die "You used a '$format_ref' type to set the 'Format', "
                    ."but currently only strings are supported!"
                    if $format_ref;
                
                print "format $format\n" if Verbose($report, 3);

                $left   = "sprintf(\"$format\", ".$left;
                $right .= ")";
            }

            # --- coloring is not supported for text ------------------
            my $color = GetOptionValue(\%options, qw (-color -c));

            print "color cannot be supported for text (color = $color)\n" if Verbose($report, 3)  &&  $color;

            # --- width / align ---------------------------------------
            my $width = GetOptionValue(\%options, qw (-width -w));
            my $align = GetOptionValue(\%options, qw (-align -a));

            $width = $report->GetDefaultColumnWidth()
                unless defined $width;

            if ($align) {
                my $align_ref = ref($align);

                die "You used a '$align_ref' type to set the 'Align', "
                    ."but currently only strings are supported!"
                    if $align_ref;

                $align = InterpreteAlignment($align);
                
                print "align $align\n" if Verbose($report, 3);
            }
            else {
                $align = '';
            }
            
            if (defined $width) {
                my $width_ref = ref($width);

                die "You used a '$width_ref' type to set the 'Width', "
                    ."but currently only strings are supported!"
                    if $width_ref;
                
                print "width $width\n" if Verbose($report, 3);

                push (@{$report->GetColumnWidthsRef()}, $width);
                
                $align = "Left" unless $align;

                $left   = "ConstLength$align($width, ".$left;
                $right .= ")";
            }
            
            # --- configure header ----------------------------

            my $header_text = GetOptionValue(\%options, qw (-header -h));
            $header_text = '' unless defined $header_text;
            push (@{$report->GetHeaderTextsRef()}, $header_text);
            
            # --- ---------------------------------------------
            # --- ---------------------------------------------
            # --- ---------------------------------------------

            # --- build cell content action ---------------------------------------------
            my $cell_action_str = $left.'$value_action->($_[0])'.$right;

            my $eval_str = 'sub { return $cell_start.'.$cell_action_str.'.$cell_end; }';

            print "### eval_str = sub { return $cell_start$cell_action_str$cell_end; }\n"
                if Verbose($report, 3);
            
            my $cell_action = eval ($eval_str);
            
            print "### ref(cell_action) ".ref($cell_action) ."\n" if Verbose($report, 3);
            
            $report->AddCellOutputAction($cell_action);
        });

    # === Configure Complete Action ====================================================
    $report2configure->SetConfigureCompleteAction(
        sub {
            my ($report,        # instance_ref
                ) = @_;

            # --- Build Header Lines -------------------------------------------
            my $start_col      = '';
            my $start_col_bold = '';

            my $additional_sep_start_chars      = '';
            my $additional_sep_end_chars        = '';
            my $additional_bold_sep_start_chars = '';
            my $additional_bold_sep_end_chars   = '';
            my $columSeparatorLength            = length($report->GetHorizontalSeparationColumnSeparator());
            
            my $cell_start_length = length($report->GetCellStart());
            my $cell_end_length   = length($report->GetCellEnd())       - $columSeparatorLength;

            my $bold_cell_start_length = length($report->GetHeaderStart());
            my $bold_cell_end_length   = length($report->GetHeaderEnd())   - $columSeparatorLength;

            # print "$cell_start_length, $cell_end_length\n";
            
            $additional_sep_start_chars = $report->GetHorizontalSeparationChar() x $cell_start_length
                if $cell_start_length > 0;
            $additional_sep_end_chars = $report->GetHorizontalSeparationChar() x $cell_end_length
                if $cell_end_length > 0;
            
            $additional_bold_sep_start_chars = $report->GetHorizontalSeparationBoldChar() x $bold_cell_start_length
                if $bold_cell_start_length > 0;
            $additional_bold_sep_end_chars = $report->GetHorizontalSeparationBoldChar() x $bold_cell_end_length
                if $bold_cell_end_length > 0;
            
            my $sep      = $report->GetHorizontalSeparationStart()
                .$additional_sep_start_chars;
            my $sep_bold = $report->GetHorizontalSeparationStart()
                .$additional_bold_sep_start_chars;

            # --- build separator lines ---
            foreach my $width (@{$report->GetColumnWidthsRef()}) {
                $sep      .= $start_col.
                    $report->GetHorizontalSeparationChar()     x $width;

                $sep_bold .= $start_col_bold.
                    $report->GetHorizontalSeparationBoldChar() x $width;

                $start_col_bold =
                    $additional_bold_sep_start_chars     x $columSeparatorLength
                    .$report->GetHorizontalSeparationColumnSeparator()
                    .$additional_bold_sep_end_chars;

                $start_col      =
                    $additional_sep_start_chars          x $columSeparatorLength
                    .$report->GetHorizontalSeparationColumnSeparator()
                    .$additional_sep_end_chars;
            }

            $sep      .= $report->GetHorizontalSeparationChar()
                .$report->GetHorizontalSeparationEnd();
            $sep_bold .= $report->GetHorizontalSeparationBoldChar()
                .$report->GetHorizontalSeparationEnd();

            $report->SetHeaderLine    ($sep);
            $report->SetSeparatorLine ($sep);
            $report->SetBoldHeaderLine($sep_bold);
            $report->SetTableEnd      ($sep_bold);
        });
    
    #===================================================================================
    #
    #  Runtime Actions, no need to speed up
    #
    #===================================================================================

    # === Header Output Action ============================================================
    $report2configure->SetHeaderOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            my $header_string = $report->GetHeaderRowStart();
            my $c = 0;
            foreach my $header_text (@{$data_ref}) {
                $header_string .= $report->GetHeaderStart()
                    .ConstLengthCenter($report->GetColumnWidthsRef()->[$c],
                                       $header_text)
                    .$report->GetHeaderEnd();
                $c++;
            }

            $header_string .= $report->GetHeaderRowEnd();

            print "### Header String:$header_string" if Verbose($report, 2);
            
            return $header_string;
        });

    # === Start Table Output Action ============================================================
    $report2configure->SetStartTableOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            return $report->GetFileStart()
                .$report->GetTableStart()
                .$report->GetBoldHeaderLine()
                .$report->GetHeaderOutput()
                .$report->GetHeaderLine();
            
        });

    # === End Table Output Action ============================================================
    $report2configure->SetEndTableOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            return $report->GetTableEnd()
                .$report->GetFileEnd();
        });

    #===================================================================================
    #
    #  Runtime Mass Data Actions, be careful, don't slow it down !!
    #
    #===================================================================================

    my $previous_data_ref = '';

    # === Row Output Action ============================================================
    $report2configure->SetRowOutputAction(
        sub {
            my ($report,        # instance_ref
                $data_ref       # data to output
                ) = @_;

            my $cell_output_actions_ref = $report->GetCellOutputActions();

            my $row_string = '';

            # --- Add something when group changes ----
            my $row_group_changes_action = $report->GetRowGroupChangesAction();
            
            $row_string .= $row_group_changes_action->($previous_data_ref, $data_ref)
                if $row_group_changes_action;

            $previous_data_ref = $data_ref;

            # --- start new row -----------------------
            $row_string .= $report->GetRowStart();
            foreach my $action (@$cell_output_actions_ref) {
                print "### action $action\n" if Verbose($report, 4);
                $row_string .= $action->($data_ref);
            }

            $row_string .= $report->GetRowEnd();

            print "### Row String:$row_string" if Verbose($report, 2);
            
            return $row_string;
        });

    return $report2configure;
}
1;

=head1 NAME

C<Report::Porf::Table::Simple::TextReportConfigurator>

Configures a Report::Porf::Table::Simple to write out ASCII- or UTF-Text tables.

Part of Perl Open Report Framework (Porf).

=head1 Documentation

See Report::Porf::Framework.pm for documentation of features and usage.

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2013 by Ralf Peine, Germany.  All rights reserved.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.6.0 or,
at your option, any later version of Perl 5 you may have available.

=head1 DISCLAIMER OF WARRANTY

This library is distributed in the hope that it will be useful,
but without any warranty; without even the implied warranty of
merchantability or fitness for a particular purpose.

=cut
